/*
** This file is part of Solarflare SolarCapture.
** 
** Copyright 2012-2017  Solarflare Communications Inc.
**                      7505 Irvine Center Drive, Irvine, CA 92618, USA
** 
** Proprietary and confidential.  All rights reserved.
** 
** Please see LICENSE.txt included in this distribution for terms of use.
*/

/**
 * \file
 * \brief sc_session: A set of threads and other objects.
 */

#ifndef __SOLAR_CAPTURE_SESSION_H__
#define __SOLAR_CAPTURE_SESSION_H__


struct sc_session;

/**
 * \brief A SolarCapture session error object returned by ::sc_session_error_get
 */
struct sc_session_error {
  char* err_msg;         /**< The error message. */
  char* err_func;        /**< The function the error occurred in. */
  char* err_file;        /**< The source file the error occurred in. */
  int   err_line;        /**< The line number the error was issued from. */
  int   err_errno;       /**< The errno for the error. */
};


/**
 * \brief Allocate a SolarCapture session.
 *
 * \param scs_out         The allocated session object is returned here
 * \param attr            Attributes for the new session
 *
 * \return 0 on success, or a negative error code.
 *
 * This function allocates a SolarCapture session.
 *
 * A session comprises a set of threads, VIs, nodes and/or other
 * SolarCapture objects.
 */
extern int sc_session_alloc(struct sc_session** scs_out,
                            const struct sc_attr* attr);


#if SC_API_VER >= 3
/**
 * \brief Prepare a SolarCapture session.
 *
 * \param scs             The session
 *
 * \return 0 on success, or a negative error code.
 *
 * Prepare the session @p scs.  This step includes finalising resource
 * allocations, preparing nodes, and starting packet capture.  Managed
 * threads are started in the "paused" state.
 *
 * Note that although packet capture is started, you may get packet loss if
 * the threads managing 'sc_vi's are not started soon afterwards.
 *
 * Call ::sc_session_go() to start the managed threads and begin packet
 * processing.
 */
extern int sc_session_prepare(struct sc_session* scs);
#endif


/**
 * \brief Start a SolarCapture session.
 *
 * \param scs             The session
 *
 * \return 0 on success, or a negative error code.
 *
 * Prepare the session @p scs (if necessary) and start the managed threads.
 * This is usually called just once, after allocating resources.  It can
 * also be called after ::sc_session_pause() to restart a paused session.
 */
extern int sc_session_go(struct sc_session* scs);


#if SC_API_VER >= 3
/**
 * \brief Pause a SolarCapture session.
 *
 * \param scs             The session
 *
 * \return 0 on success, or a negative error code.
 *
 * Pause the threads managed by session @p scs.
 */
extern int sc_session_pause(struct sc_session* scs);
#endif


#if SC_API_VER >= 3
/**
 * \brief Returns an error from a SolarCapture session.
 *
 * \param scs             The session
 *
 * \return A pointer to a ::sc_session_error struct representing the error
 * encountered by session @p scs. The caller should pass the pointer to
 * ::sc_session_error_free once once done with it.\n
 * If no error has occurred, this function returns NULL.
 */
extern struct sc_session_error* sc_session_error_get(struct sc_session* scs);
#endif


#if SC_API_VER >= 3
/**
 * \brief Frees an error object
 *
 * \param scs             The session
 * \param err             The error
 *
 * Frees a ::sc_session_error pointer returned by ::sc_session_error_get.
 */
extern void sc_session_error_free(struct sc_session* scs, struct sc_session_error* err);
#endif

#endif  /* __SOLAR_CAPTURE_SESSION_H__ */
/** @} */
